<?php
/**
 * Plugin Name: Advanced Product Labels for WooCommerce
 * Plugin URI: https://wordpress.org/plugins/advanced-product-labels-for-woocommerce/
 * Description: Promote your products! Show “Free Shipping” or other special attributes for your products.
 * Version: 1.0.7
 * Author: BeRocket
 * Requires at least: 4.0
 * Author URI: http://berocket.com
 * Text Domain: BeRocket_products_label_domain
 * Domain Path: /languages/
 */
define( "BeRocket_products_label_version", '1.0.7' );
define( "BeRocket_products_label_domain", 'BeRocket_products_label_domain'); 
define( "products_label_TEMPLATE_PATH", plugin_dir_path( __FILE__ ) . "templates/" );
load_plugin_textdomain('BeRocket_products_label_domain', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/');
require_once(plugin_dir_path( __FILE__ ).'includes/functions.php');
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

class BeRocket_products_label {

    public static $info = array( 
        'id'        => 18,
        'version'   => BeRocket_products_label_version,
        'plugin'    => '',
        'slug'      => '',
        'key'       => '',
        'name'      => ''
    );

    /**
     * Defaults values
     */
    public static $defaults = array(
        'disable_labels'    => '0',
        'disable_plabels'   => '0',
        'disable_ppage'     => '0',
        'custom_css'        => '.product .images {position: relative;}',
        'script'            => '',
        'plugin_key'        => '',
    );
    public static $values = array(
        'settings_name' => 'br-products_label-options',
        'option_page'   => 'br-products_label',
        'premium_slug'  => 'woocommerce-advanced-products-labels',
    );
    
    function __construct () {
        register_activation_hook(__FILE__, array( __CLASS__, 'activation' ) );
        register_uninstall_hook(__FILE__, array( __CLASS__, 'deactivation' ) );

        if ( ( is_plugin_active( 'woocommerce/woocommerce.php' ) || is_plugin_active_for_network( 'woocommerce/woocommerce.php' ) ) && 
            br_get_woocommerce_version() >= 2.1 ) {
            $options = self::get_option();
            
            add_action ( 'init', array( __CLASS__, 'init' ) );
            add_action ( 'wp_head', array( __CLASS__, 'set_styles' ) );
            add_action ( 'admin_init', array( __CLASS__, 'admin_init' ) );
            add_action ( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue_scripts' ) );
            add_action ( 'admin_menu', array( __CLASS__, 'options' ) );
            add_action ( "wp_ajax_br_products_label_settings_save", array ( __CLASS__, 'save_settings' ) );
            add_action ( 'woocommerce_before_shop_loop_item_title', array( __CLASS__, 'set_all_label'), 20 );
            add_action ( 'product_of_day_before_thumbnail_widget', array( __CLASS__, 'set_image_label'), 20 );
            add_action ( 'product_of_day_before_title_widget', array( __CLASS__, 'set_label_label'), 20 );
            add_action ( 'lgv_advanced_after_img', array( __CLASS__, 'set_all_label'), 20 );
            if( ! @ $options['disable_ppage'] ) {
                add_action( 'woocommerce_product_thumbnails', array( __CLASS__, 'set_all_label'), 10 );
                add_action( 'woocommerce_product_thumbnails', array( __CLASS__, 'move_labels_from_zoom'), 20 );
            }
            add_filter( 'plugin_row_meta', array( __CLASS__, 'plugin_row_meta' ), 10, 2 );
            $plugin_base_slug = plugin_basename( __FILE__ );
            add_filter( 'plugin_action_links_' . $plugin_base_slug, array( __CLASS__, 'plugin_action_links' ) );
        }
    }
    public static function plugin_action_links($links) {
		$action_links = array(
			'settings' => '<a href="' . admin_url( 'admin.php?page='.self::$values['option_page'] ) . '" title="' . __( 'View Plugin Settings', 'BeRocket_products_label_domain' ) . '">' . __( 'Settings', 'BeRocket_products_label_domain' ) . '</a>',
		);
		return array_merge( $action_links, $links );
    }
    public static function plugin_row_meta($links, $file) {
        $plugin_base_slug = plugin_basename( __FILE__ );
        if ( $file == $plugin_base_slug ) {
			$row_meta = array(
				'docs'    => '<a href="http://berocket.com/docs/plugin/'.self::$values['premium_slug'].'" title="' . __( 'View Plugin Documentation', 'BeRocket_products_label_domain' ) . '" target="_blank">' . __( 'Docs', 'BeRocket_products_label_domain' ) . '</a>',
				'premium'    => '<a href="http://berocket.com/product/'.self::$values['premium_slug'].'" title="' . __( 'View Premium Version Page', 'BeRocket_products_label_domain' ) . '" target="_blank">' . __( 'Premium Version', 'BeRocket_products_label_domain' ) . '</a>',
			);

			return array_merge( $links, $row_meta );
		}
		return (array) $links;
    }
    public static function init () {
        $options = self::get_option();
        wp_enqueue_script("jquery");
        wp_register_style( 'font-awesome', plugins_url( 'css/font-awesome.min.css', __FILE__ ) );
        wp_enqueue_style( 'font-awesome' );
        register_post_type( "br_labels",
			array(
				'labels' => array(
					'name'               => __( 'Advanced Label', 'BeRocket_products_label_domain' ),
					'singular_name'      => __( 'Advanced Label', 'BeRocket_products_label_domain' ),
					'menu_name'          => _x( 'Advanced Labels', 'Admin menu name', 'BeRocket_products_label_domain' ),
					'add_new'            => __( 'Add Label', 'BeRocket_products_label_domain' ),
					'add_new_item'       => __( 'Add New Label', 'BeRocket_products_label_domain' ),
					'edit'               => __( 'Edit', 'BeRocket_products_label_domain' ),
					'edit_item'          => __( 'Edit Label', 'BeRocket_products_label_domain' ),
					'new_item'           => __( 'New Label', 'BeRocket_products_label_domain' ),
					'view'               => __( 'View Labels', 'BeRocket_products_label_domain' ),
					'view_item'          => __( 'View Label', 'BeRocket_products_label_domain' ),
					'search_items'       => __( 'Search Advanced Labels', 'BeRocket_products_label_domain' ),
					'not_found'          => __( 'No Advanced Labels found', 'BeRocket_products_label_domain' ),
					'not_found_in_trash' => __( 'No Advanced Labels found in trash', 'BeRocket_products_label_domain' ),
				),
				'description'     => __( 'This is where you can add advanced labels.', 'BeRocket_products_label_domain' ),
				'public'          => true,
				'show_ui'         => true,
				'capability_type' => 'post',
				'publicly_queryable'  => false,
				'exclude_from_search' => true,
				'show_in_menu'        => 'edit.php?post_type=product',
				'hierarchical'        => false,
				'rewrite'             => false,
				'query_var'           => false,
				'supports'            => array( 'title' ),
				'show_in_nav_menus'   => false,
			)
		);
        wp_register_style( 'berocket_products_label_style', 
            plugins_url( 'css/frontend.css', __FILE__ ), 
            "", 
            BeRocket_products_label_version );
        wp_enqueue_style( 'berocket_products_label_style' );
    }
    /**
     * Function set styles in wp_head WordPress action
     *
     * @return void
     */
    public static function set_styles () {
        $options = self::get_option();
        echo '<style>'.$options['custom_css'].'</style>';
    }
    /**
     * Load template
     *
     * @access public
     *
     * @param string $name template name
     *
     * @return void
     */
    public static function br_get_template_part( $name = '' ) {
        $template = '';

        // Look in your_child_theme/woocommerce-products_label/name.php
        if ( $name ) {
            $template = locate_template( "woocommerce-products_label/{$name}.php" );
        }

        // Get default slug-name.php
        if ( ! $template && $name && file_exists( products_label_TEMPLATE_PATH . "{$name}.php" ) ) {
            $template = products_label_TEMPLATE_PATH . "{$name}.php";
        }

        // Allow 3rd party plugin filter template file from their plugin
        $template = apply_filters( 'products_label_get_template_part', $template, $name );

        if ( $template ) {
            load_template( $template, false );
        }
    }

    public static function admin_enqueue_scripts() {
        if ( function_exists( 'wp_enqueue_media' ) ) {
            wp_enqueue_media();
        } else {
            wp_enqueue_style( 'thickbox' );
            wp_enqueue_script( 'media-upload' );
            wp_enqueue_script( 'thickbox' );
        }
    }

    /**
     * Function adding styles/scripts and settings to admin_init WordPress action
     *
     * @access public
     *
     * @return void
     */
    public static function admin_init () {
        wp_enqueue_script( 'berocket_products_label_admin', plugins_url( 'js/admin.js', __FILE__ ), array( 'jquery' ), BeRocket_products_label_version );
        wp_register_style( 'berocket_products_label_admin_style', plugins_url( 'css/admin.css', __FILE__ ), "", BeRocket_products_label_version );
        wp_enqueue_style( 'berocket_products_label_admin_style' );
        wp_enqueue_script( 'berocket_products_label_colpick', plugins_url( 'js/colpick.js', __FILE__ ), array( 'jquery' ), BeRocket_products_label_version );
        wp_register_style( 'berocket_products_label_colpick_style', plugins_url( 'css/colpick.css', __FILE__ ), "", BeRocket_products_label_version );
        wp_enqueue_style( 'berocket_products_label_colpick_style' );
        wp_enqueue_script( 'berocket_global_admin', plugins_url( 'js/admin_global.js', __FILE__ ), array( 'jquery' ) );
        wp_localize_script( 'berocket_global_admin', 'berocket_global_admin', array(
            'security' => wp_create_nonce("search-products")
        ) );
        add_filter( 'bulk_actions-edit-br_labels', array( __CLASS__, 'bulk_actions_edit' ) );
        add_filter( 'views_edit-br_labels', array( __CLASS__, 'views_edit' ) );
        add_filter( 'manage_edit-br_labels_columns', array( __CLASS__, 'manage_edit_columns' ) );
        add_action( 'manage_br_labels_posts_custom_column', array( __CLASS__, 'columns_replace' ), 2 );
        add_action( 'add_meta_boxes', array( __CLASS__, 'add_meta_boxes' ) );
        add_action( 'save_post', array( __CLASS__, 'wc_save_product' ) );
    }
    public static function wc_save_product( $product_id ) {
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return;
        }
        if ( isset( $_POST['br_label'] ) ) {
            if( ! isset($_POST['br_label']['color_use']) ) {
                $_POST['br_label']['color_use'] = 0;
            }
            if( @ $_POST['br_label']['padding_top'] < -10 || @ $_POST['br_label']['padding_top'] > 200 ) {
                $_POST['br_label']['padding_top'] = -10;
            }
            update_post_meta( $product_id, 'br_label', $_POST['br_label'] );
        } else {
            delete_post_meta( $product_id, 'br_label' );
        }
    }
    public static function bulk_actions_edit ( $actions ) {
        unset( $actions['edit'] );
        return $actions;
    }
    public static function views_edit ( $view ) {
        unset( $view['publish'], $view['private'], $view['future'] );
        return $view;
    }
    public static function manage_edit_columns ( $columns ) {
        $columns = array();
        $columns["cb"]   = '<input type="checkbox" />';
        $columns["name"] = __( "Label Name", 'BeRocket_products_label_domain' );
        $columns["products"] = __( "Label text", 'BeRocket_products_label_domain' );
        $columns["data"] = __( "Position", 'BeRocket_products_label_domain' );
        return $columns;
    }
    public static function columns_replace ( $column ) {
        global $post;
        switch ( $column ) {
            case "name":

                $edit_link = get_edit_post_link( $post->ID );
                $title = '<a class="row-title" href="' . $edit_link . '">' . _draft_or_post_title() . '</a>';

                echo '<strong>' . $title . '</strong>';

                // Get actions
                $actions = array();

                $post_type_object = get_post_type_object( $post->post_type );

                if ( current_user_can( $post_type_object->cap->delete_post, $post->ID ) ) {
                    if ( 'trash' == $post->post_status )
                        $actions['untrash'] = "<a title='" . __( 'Restore this item from the Trash', 'BeRocket_products_label_domain' ) . "' href='" . wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=untrash', $post->ID ) ), 'untrash-post_' . $post->ID ) . "'>" . __( 'Restore', 'BeRocket_products_label_domain' ) . "</a>";
                    elseif ( EMPTY_TRASH_DAYS )
                        $actions['trash'] = "<a class='submitdelete' title='" . __( 'Move this item to the Trash', 'BeRocket_products_label_domain' ) . "' href='" . get_delete_post_link( $post->ID ) . "'>" . __( 'Trash', 'BeRocket_products_label_domain' ) . "</a>";
                    if ( 'trash' == $post->post_status || ! EMPTY_TRASH_DAYS )
                        $actions['delete'] = "<a class='submitdelete' title='" . __( 'Delete this item permanently', 'BeRocket_products_label_domain' ) . "' href='" . get_delete_post_link( $post->ID, '', true ) . "'>" . __( 'Delete Permanently', 'BeRocket_products_label_domain' ) . "</a>";
                }

                $actions = apply_filters( 'post_row_actions', $actions, $post );

                echo '<div class="row-actions">';

                $i = 0;
                $action_count = count( $actions );

                foreach ( $actions as $action => $link ) {
                    ( $i == $action_count - 1 ) ? $sep = '' : $sep = ' | ';
                    echo '<span class="' . sanitize_html_class( $action ) . '">' . $link . $sep . '</span>';
                    $i++;
                }
                echo '</div>';
                
                break;
            case "products":
                $label_type = get_post_meta( $post->ID, 'br_label', true );
                if( isset($label_type['text']) ) {
                    echo $label_type['text'];
                }
                break;
            case "data":
                $label_type = get_post_meta( $post->ID, 'br_label', true );
                $position = array('left' => __('Left', 'BeRocket_products_label_domain'), 'center' => __('Center', 'BeRocket_products_label_domain'), 'right' => __('Right', 'BeRocket_products_label_domain'));
                $type = array('image' => __('On image', 'BeRocket_products_label_domain'), 'label' => __('Label', 'BeRocket_products_label_domain'));
                if( isset($label_type['position']) && isset($label_type['type']) ) {
                    echo $type[$label_type['type']].' ( '.$position[$label_type['position']].' )';
                }
                break;
        }
    }
    public static function add_meta_boxes () {
        add_meta_box( 'submitdiv', __( 'Save label content', 'BeRocket_products_label_domain' ), array( __CLASS__, 'meta_box' ), 'br_labels', 'side', 'high' );
        add_meta_box( 'labels_setup', __( 'Label settings', 'BeRocket_products_label_domain' ), array( __CLASS__, 'meta_box_settings' ), 'br_labels', 'normal', 'high' );
    }
    public static function meta_box($post) {
        ?>
        <div class="submitbox" id="submitpost">

            <div id="minor-publishing">
                <div id="major-publishing-actions">
                    <div id="delete-action">
                        <?php
                        global $pagenow;
                        if( in_array( $pagenow, array( 'post-new.php' ) ) ) {
                        } else {
                            if ( current_user_can( "delete_post", $post->ID ) ) {
                                if ( ! EMPTY_TRASH_DAYS )
                                    $delete_text = __( 'Delete Permanently', 'BeRocket_products_label_domain' );
                                else
                                    $delete_text = __( 'Move to Trash', 'BeRocket_products_label_domain' );
                                ?>
                                <a class="submitdelete deletion" href="<?php echo esc_url( get_delete_post_link( $post->ID ) ); ?>"><?php echo esc_attr( $delete_text ); ?></a>
                            <?php 
                            }
                        } ?>
                    </div>

                    <div id="publishing-action">
                        <span class="spinner"></span>
                        <input type="submit" class="button button-primary tips" name="publish" value="<?php _e( 'Save Label', 'BeRocket_products_label_domain' ); ?>" data-tip="<?php _e( 'Save/update notice', 'BeRocket_products_label_domain' ); ?>" />
                    </div>
                    <div class="clear"></div>
                </div>
            </div>
        </div>
        <?php
    }
    public static function meta_box_settings($post) {
        include products_label_TEMPLATE_PATH . "label.php";
    }
    public static function move_labels_from_zoom() {
        ?>
        <script>
            jQuery(".woocommerce-product-gallery .br_alabel").each(function(i, o) {
                jQuery(o).hide().parents(".woocommerce-product-gallery").append(jQuery(o));
            });
            galleryReadyCheck = setInterval(function() {
                if( jQuery(".woocommerce-product-gallery .woocommerce-product-gallery__trigger").length > 0 ) {
                    clearTimeout(galleryReadyCheck);
                    jQuery(".woocommerce-product-gallery .br_alabel").each(function(i, o) {
                        jQuery(o).show().parents(".woocommerce-product-gallery").append(jQuery(o));
                    });
                }
            }, 250);
        </script>
        <?php
    }
    public static function set_all_label() {
        self::set_label();
    }
    public static function set_image_label() {
        self::set_label('image');
    }
    public static function set_label_label() {
        echo '<div>';
        self::set_label('label');
        echo '<div style="clear:both;"></div></div>';
    }
    public static function set_label($type = TRUE) {
        global $product;
        $product_post = br_wc_get_product_post($product);
        $options = self::get_option();
        if( ! $options['disable_plabels'] ) {
            $label_type = get_post_meta( $product_post->ID, 'br_label', true );
            self::show_label_on_product($label_type, $product);
        }
        if( ! $options['disable_labels'] ) {
            $args = array(
                'posts_per_page'   => -1,
                'offset'           => 0,
                'category'         => '',
                'category_name'    => '',
                'orderby'          => 'date',
                'order'            => 'DESC',
                'include'          => '',
                'exclude'          => '',
                'meta_key'         => '',
                'meta_value'       => '',
                'post_type'        => 'br_labels',
                'post_mime_type'   => '',
                'post_parent'      => '',
                'author'	   => '',
                'post_status'      => 'publish',
                'suppress_filters' => false 
            );
            $posts_array = get_posts( $args );
            foreach($posts_array as $label) {
                $br_label = get_post_meta( $label->ID, 'br_label', true );
                if( $type === TRUE || $type == $br_label['type'] ) {
                    if( ! isset($br_label['data']) || self::check_label_on_post($br_label['data'], $product) ) {
                        self::show_label_on_product($br_label, $product);
                    }
                }
            }
        }
    }
    public static function show_label_on_product($br_label, $product) {
        if( @ $br_label['text'] && @ $br_label['content_type'] == 'text' 
        || @ $br_label['content_type'] == 'sale_p') {
            if ( $br_label['color'][0] != '#' ) {
                $br_label['color'] = '#'.$br_label['color'];
            }
            if ( isset($br_label['font_color']) && $br_label['font_color'][0] != '#' ) {
                $br_label['font_color'] = '#'.$br_label['font_color'];
            }
            if( @ $br_label['content_type'] == 'sale_p' ) {
                $br_label['text'] = '';
                if( $product->is_on_sale() ) {
                    $price_ratio = false;
                    $product_sale = br_wc_get_product_attr($product, 'sale_price');
                    $product_regular = br_wc_get_product_attr($product, 'regular_price');
                    if( ! empty($product_sale) && $product_sale != $product_regular ) {
                        $price_ratio = $product_sale / $product_regular;
                    }
                    if( $product->has_child() ) {
                        foreach($product->get_children() as $child_id) {
                            $child = br_wc_get_product_attr($product, 'child', $child_id);
                            $child_sale = br_wc_get_product_attr($child, 'sale_price');
                            $child_regular = br_wc_get_product_attr($child, 'regular_price');
                            if( ! empty($child_sale) && $child_sale != $child_regular ) {
                                $price_ratio2 = $child_sale / $child_regular;
                                if( $price_ratio === false || $price_ratio2 < $price_ratio ) {
                                    $price_ratio = $price_ratio2;
                                }
                            }
                        }
                    }
                    if( $price_ratio !== false ) {
                        $price_ratio = ($price_ratio * 100);
                        $price_ratio = number_format($price_ratio, 0, '', '');
                        $price_ratio = $price_ratio * 1;
                        $br_label['text'] = (100 - $price_ratio)."%";
                        if( @ $br_label['discount_minus'] ) {
                            $br_label['text'] = '-'.$br_label['text'];
                        }
                    }
                }
            }
            if( @ $br_label['text'] ) {
                echo '<div class="br_alabel br_alabel_'.$br_label['type'].' br_alabel_'. @ $br_label['content_type'] . ' br_alabel_'.$br_label['position'].'">';
                echo '<span style="'.
                'background-color:'.@ $br_label['color'].';'.
                'color:'.@ $br_label['font_color'].';">'.$br_label['text'].'</span>';
                echo '</div>';
            }
        }
    }
    public static function check_label_on_post($label_data, $product) {
        $show_label = false;
        $product_id = br_wc_get_product_id($product);
        $product_post = br_wc_get_product_post($product);
        foreach($label_data as $label) {
            $show_label = false;
            foreach($label as $condition) {
                $show_label = false;
                if( $condition['type'] == 'product' ) {
                    $show_label = in_array($product_id, $condition['product']);
                    if( $condition['equal'] == 'not_equal' ) {
                        $show_label = ! $show_label;
                    }
                } elseif( $condition['type'] == 'category' ) {
                    $terms = get_the_terms( $product_id, 'product_cat' );
                    if( is_array( $terms ) ) {
                        foreach( $terms as $term ) {
                            if( $term->term_id == $condition['category']) {
                                $show_label = true;
                                break;
                            }
                        }
                    }
                    if( $condition['equal'] == 'not_equal' ) {
                        $show_label = ! $show_label;
                    }
                } elseif( $condition['type'] == 'sale' ) {
                    $show_label = $product->is_on_sale();
                    if( $condition['sale'] == 'no' ) {
                        $show_label = ! $show_label;
                    }
                } elseif( $condition['type'] == 'bestsellers' ) {
                    $args = array(
                        'post_type' 			=> 'product',
                        'post_status' 			=> 'publish',
                        'ignore_sticky_posts'   => 1,
                        'posts_per_page'		=> $condition['bestsellers'],
                        'meta_key' 		 		=> 'total_sales',
                        'orderby' 		 		=> 'meta_value_num',
                        'meta_query' 			=> array(
                            array(
                                'key' 		=> '_visibility',
                                'value' 	=> array( 'catalog', 'visible' ),
                                'compare' 	=> 'IN'
                            )
                        )
                    );
                    $posts = get_posts( $args );
                    if( is_array( $posts ) ) {
                        foreach($posts as $post) {
                            if( $product_post->ID == $post->ID ) {
                                $show_label = true;
                                break;
                            }
                        }
                    }
                } elseif( $condition['type'] == 'price' ) {
                    $product_price = br_wc_get_product_attr($product, 'price');
                    $show_label = $product_price >= $condition['price']['from'] && $product_price <= $condition['price']['to'];
                    if( ! $show_label && $product->has_child() ) {
                        foreach($product->get_children() as $child_id) {
                            $child = br_wc_get_product_attr($product, 'child', $child_id);
                            $child_price = br_wc_get_product_attr($child, 'price');
                            $show_label = $child_price >= $condition['price']['from'] && $child_price <= $condition['price']['to'];
                            if( $show_label ) {
                                break;
                            }
                        }
                    }
                    if( $condition['equal'] == 'not_equal' ) {
                        $show_label = ! $show_label;
                    }
                } elseif( $condition['type'] == 'stockstatus' ) {
                    $show_label = $product->is_in_stock();
                    if( $condition['stockstatus'] == 'out_of_stock' ) {
                        $show_label = ! $show_label;
                    }
                } elseif( $condition['type'] == 'totalsales' ) {
                    $total_sales = get_post_meta( $product_id, 'total_sales', true );
                    if( $condition['equal'] == 'equal' ) {
                        $show_label = $condition['totalsales'] == $total_sales;
                    } elseif( $condition['equal'] == 'not_equal' ) {
                        $show_label = $condition['totalsales'] != $total_sales;
                    } elseif( $condition['equal'] == 'equal_less' ) {
                        $show_label = $condition['totalsales'] >= $total_sales;
                    } elseif( $condition['equal'] == 'equal_more' ) {
                        $show_label = $condition['totalsales'] <= $total_sales;
                    }
                } else {
                    $show_label = false;
                }
                if( !$show_label ) {
                    break;
                }
            }
            if( $show_label ) {
                break;
            }
        }
        return $show_label;
    }
    /**
     * Function add options button to admin panel
     *
     * @access public
     *
     * @return void
     */
    public static function options() {
        add_submenu_page( 'woocommerce', __('Products Labels settings', 'BeRocket_products_label_domain'), __('Products Labels', 'BeRocket_products_label_domain'), 'manage_options', 'br-products_label', array(
            __CLASS__,
            'option_form'
        ) );
    }
    /**
     * Function add options form to settings page
     *
     * @access public
     *
     * @return void
     */
    public static function option_form() {
        $plugin_info = get_plugin_data(__FILE__, false, true);
        include products_label_TEMPLATE_PATH . "settings.php";
    }
    /**
     * Function set default settings to database
     *
     * @return void
     */
    public static function activation () {
        $options = self::get_option();
        foreach ( self::$defaults as $key2 => $val2 ) {
            $options[ $key2 ] = $val2;
        }
        update_option( self::$values['settings_name'], $options );
    }
    /**
     * Function remove settings from database
     *
     * @return void
     */
    public static function deactivation () {
        delete_option( self::$values['settings_name'] );
    }
    public static function save_settings () {
        if( current_user_can( 'manage_options' ) ) {
            if( isset($_POST[self::$values['settings_name']]) ) {
                update_option( self::$values['settings_name'], self::sanitize_option($_POST[self::$values['settings_name']]) );
                echo json_encode($_POST[self::$values['settings_name']]);
            }
        }
        wp_die();
    }

    public static function sanitize_option( $input ) {
        $default = self::$defaults;
        $result = self::recursive_array_set( $default, $input );
        return $result;
    }
    public static function recursive_array_set( $default, $options ) {
        $result = array();
        foreach( $default as $key => $value ) {
            if( array_key_exists( $key, $options ) ) {
                if( is_array( $value ) ) {
                    if( is_array( $options[$key] ) ) {
                        $result[$key] = self::recursive_array_set( $value, $options[$key] );
                    } else {
                        $result[$key] = self::recursive_array_set( $value, array() );
                    }
                } else {
                    $result[$key] = $options[$key];
                }
            } else {
                if( is_array( $value ) ) {
                    $result[$key] = self::recursive_array_set( $value, array() );
                } else {
                    $result[$key] = '';
                }
            }
        }
        foreach( $options as $key => $value ) {
            if( ! array_key_exists( $key, $result ) ) {
                $result[$key] = $value;
            }
        }
        return $result;
    }
    public static function get_option() {
        $options = get_option( self::$values['settings_name'] );
        if ( @ $options && is_array ( $options ) ) {
            $options = array_merge( self::$defaults, $options );
        } else {
            $options = self::$defaults;
        }
        return $options;
    }
}

new BeRocket_products_label;
